#include <ginac/ginac.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <boost/algorithm/string.hpp>

using namespace GiNaC;

int main (int argc, char** argv) {
    bool verbose = false;
    for (int ii=1; ii<argc; ii++) { if(std::strcmp(argv[ii], "-v")==0) { verbose=true; } }
    std::string line;
    std::list<std::string> names;
    std::vector<std::vector<std::string> > connections;
    std::vector<std::string> lnames;
    while (getline(std::cin, line)) {
        boost::trim_right(line);
        boost::split(lnames, line, boost::is_any_of("\t "));
        //std::for_each (lnames.begin(), lnames.end(), [](std::string nn){ names.push_back(nn); } );
        for (size_t ii=0; ii < lnames.size(); ii++) {
            names.push_back(lnames[ii]);
        }
        connections.push_back(lnames);
    }
    names.sort();
    names.unique();
    std::map<std::string, size_t> names_map;
    int count = 0;
    printf("Building the names_map\n");
    for (std::list<std::string>::iterator prot_name=names.begin(); prot_name!=names.end(); prot_name++) {
        if(verbose) { printf("%s ", prot_name->c_str()); }
        names_map.insert( std::pair<std::string, size_t>(*prot_name, count++) );
    }
    if (verbose) { printf("\n"); }

    matrix adm(names.size(), names.size());
    std::string name0, name1;
    printf("Building the adjacency matrix\n");
    for (size_t ii=0; ii < connections.size(); ii++) {
        name0 = connections[ii][0];
        name1 = connections[ii][1];
        adm( names_map[name0], names_map[name1] ) = symbol("r_" + name0 + "_" + name1);
    }
    for (size_t ii=0; ii < adm.cols(); ii++) {
        adm(ii, ii) = -1;
        if (verbose) {
            for (size_t jj=0; jj < adm.cols(); jj++) {
                std::cerr << std::setw(16) << adm(ii, jj) << "\t";
            }
            std::cerr << std::endl;
        }
    }
    
    time_t start;
    printf("Trying inversion\n");
    std::time(&start);
    matrix inverted = adm.inverse();
    double seconds = std::difftime(time(NULL), start);
    printf("%.f seconds to invert\n", seconds);
    printf("Trying copying\n");
    inverted = matrix(inverted);
    printf("Trying multiplication by -1\n");
    inverted = inverted.mul_scalar(-1);
    printf("Success\n");

}

